part of dartcord;

class _DiscordEndpoints {
  static const String getGatewayWithShards = "/gateway/bot";
  static const String getGatewayNoShards = "/gateway";
  static String user(String id) => "/users/" + id;
  static String invite(String id) => "/invites/" + id;
  static String channel(String id) => "/channels/$id";
  static String guild(String id) => "/guilds/$id";
  static String guildRoles(String id) => guild(id) + "/roles";
  static String guildRole(String guildId, String roleId) => guildRoles(guildId) + "/$roleId";
  static String guildBans(String id) => guild(id) + "/bans";
  static String guildBan(String guildId, String userId) => guildBans(guildId) + "/$userId";
  static String guildMembers(String guildId) => guild(guildId) + "/members";
  static String guildMember(String guildId, String userId) => guildMembers(guildId) + "/$userId";
  static String currentUserGuilds() => user("@me") + "/guilds";
  static String currentUserChannels() => user("@me") + "/channels";
  static String guildChannels(String id) => guild(id) + "/channels";
  static String currentUserGuild(String guildId) => currentUserGuilds() + "/$guildId";
  static String pruneGuild(String guildId) => guild(guildId) + "/prune";
  static String channelMessages(String channelId) => channel(channelId) + "/messages";
  static String channelMessage(String channelId, String messageId) => channelMessages(channelId) + "/" + messageId;
  static String channelTyping(String id) => _DiscordEndpoints.channel(id) + "/typing";
  static String auditLogs(String id) => guild(id) + "/audit-logs";
  static String guildWebhooks(String guildId) => guild(guildId) + '/webhooks';
  static String channelWebhooks(String channelId) => channel(channelId) + '/webhooks';
  static String webhooks() => "/webhooks";
  static String webhook(String id, String token) => webhooks() + "/$id/$token";
}

class _CDNEndpoints {
  static String userAvatar(String userId, String avatarFileName, [String format = 'jpg']) => "${_Constants.host}/users/${userId}/avatars/${avatarFileName}.${format}";
}

class _OAuth2Endpoints {
  static const String oAuth2Base = "/oauth2";
  static const String getCurrentApplication = oAuth2Base + "/applications/@me";
  static String authorize(String clientId, [String scope='bot']) => '/oauth2/authorize?client_id=$clientId&scope=$scope';
}

class _OpCodes {
  static const int dispatch = 0;
  static const int heartbeat = 1;
  static const int identify = 2;
  static const int statusUpdate = 3;
  static const int voiceStateUpdate = 4;
  static const int voiceGuildPing = 5;
  static const int resume = 6;
  static const int reconnect = 7;
  static const int requestGuildMembers = 8;
  static const int invalidSession = 9;
  static const int hello = 10;
  static const int heartbeatAck = 11;
  static const int guildSync = 12;
}

class _VoiceOpCodes {
  static const int identify = 0;
  static const int selectProtocol = 1;
  static const int ready = 2;
  static const int heartbeat = 3;
  static const int sessionDescription = 4;
  static const int speaking = 5;
}

/// The client constants.
class _Constants {
  static const String host = "discordapp.com";
  static const String baseUri = "/api/v6";
  static const String clientVersion = "0.1.0";
  static const String library = "dartcord";
  static const String repo = "https://github.com/jacksonrakena/dartcord";

  /// The gateway OP codes.
  static const Map<String, int> opCodes = const <String, int>{
    "DISPATCH": 0,
    "HEARTBEAT": 1,
    "IDENTIFY": 2,
    "STATUS_UPDATE": 3,
    "VOICE_STATE_UPDATE": 4,
    "VOICE_GUILD_PING": 5,
    "RESUME": 6,
    "RECONNECT": 7,
    "REQUEST_GUILD_MEMBERS": 8,
    "INVALID_SESSION": 9,
    "HELLO": 10,
    "HEARTBEAT_ACK": 11,
    "GUILD_SYNC": 12
  };

  /// The gateway OP codes for voice.
  static const Map<String, int> voiceOpCodes = const <String, int>{
    "IDENTIFY": 0,
    "SELECT_PROTOCOL": 1,
    "READY": 2,
    "HEARTBEAT": 3,
    "SESSION_DESCRIPTION": 4,
    "SPEAKING": 5
  };

  /// The permission bits.
  static const Map<String, int> permissions = const <String, int>{
    "CREATE_INSTANT_INVITE": 1 << 0,
    "KICK_MEMBERS": 1 << 1,
    "BAN_MEMBERS": 1 << 2,
    "ADMINISTRATOR": 1 << 3,
    "MANAGE_CHANNELS": 1 << 4,
    "MANAGE_GUILD": 1 << 5,
    "READ_MESSAGES": 1 << 10,
    "SEND_MESSAGES": 1 << 11,
    "SEND_TTS_MESSAGES": 1 << 12,
    "MANAGE_MESSAGES": 1 << 13,
    "EMBED_LINKS": 1 << 14,
    "ATTACH_FILES": 1 << 15,
    "READ_MESSAGE_HISTORY": 1 << 16,
    "MENTION_EVERYONE": 1 << 17,
    "EXTERNAL_EMOJIS": 1 << 18,
    "CONNECT": 1 << 20,
    "SPEAK": 1 << 21,
    "MUTE_MEMBERS": 1 << 22,
    "DEAFEN_MEMBERS": 1 << 23,
    "MOVE_MEMBERS": 1 << 24,
    "USE_VAD": 1 << 25,
    "CHANGE_NICKNAME": 1 << 26,
    "MANAGE_NICKNAMES": 1 << 27,
    "MANAGE_ROLES_OR_PERMISSIONS": 1 << 28,
    "MANAGE_WEBHOOKS": 1 << 29,
    "MANAGE_EMOJIS": 1 << 30
  };
}
