part of dartcord;

/// The WS manager for the client.
class _SocketManager {
  /// Whether this WebSocket is a bot.
  bool bot = false;

  /// The base WebSocket URL.
  String gateway;

  /// The client that the WS manager belongs to.
  Client client;

  /// The recommended amount of shards to connect with.
  int recommendedShards;

  /// Makes a new WS manager.
  _SocketManager(this.client) {
    this.client._http.headers['Authorization'] =
    "${this.client._options.tokenType} ${client._token}";
  }

  Future createAll() async {
    var response = await this.client._http.send("GET", _DiscordEndpoints.getGatewayWithShards, beforeReady: true);
    this.bot = true;
    this.gateway = response.body.asJson()['url'];
    this.recommendedShards = response.body.asJson()['shards'];

    if (client._options.autoShard) {
        if (this.client._options.shardCount == 1 &&
            this.client._options.shardIds == const [0]) {
          this.client._options.shardIds = [];
          this.client._options.shardCount = this.recommendedShards;
          for (int i = 0; i < client._options.shardCount; i++) {
            await launchClient(i);
          }
        } else {
          for (int shardId in this.client._options.shardIds) {
            await launchClient(shardId);
          }
        }
        for (int i = 0; i < client._options.shardCount; i++) {
          await connectShard(i);
        }
    }
  }

  /// Configures and creates a shard.
  Future launchClient(int shardId) async {
    _DiscordSocket shard = new _DiscordSocket._new(this, shardId);
    this.client.shards[shard.id] = shard;
    this.client._options.shardIds.add(shardId);

    shard.onReady.stream.listen((_DiscordSocket s) {
      if (!client.ready) {
        testReady();
      }
    });
  }

  /// Closes all shards and destroys this socket.
  Future<Null> close() async {
    this.client.shards.forEach((int id, _DiscordSocket shard) async {
      await shard._socket.close();
    });
    return null;
  }

  /// Tells a shard to connect.
  Future connectShard(int index) async {
    await this.client.shards.values.toList()[index]._connect(true, true);
  }

  /// Tests if all shards are ready, and raises the ReadyEvent if true.
  void testReady() {
    bool match = true;
    client.guilds.forEach((String id, Guild o) {
      if (client._options.forceFetchMembers) {
        if (o == null || o.members.length != o.memberCount) match = false;
      } else {
        if (o == null) match = false;
      }
    });

    bool match2 = true;
    this.client.shards.forEach((int id, _DiscordSocket s) {
      if (!s.ready) match = false;
    });

    if (match && match2) {
      client.ready = true;
      client._startTime = new DateTime.now();
      if (client.user.bot) {
        client._http
            .send('GET', _OAuth2Endpoints.getCurrentApplication, beforeReady: true)
            .then((w_transport.Response r) {
          client.app = new ClientOAuth2Application._new(
              client, r.body.asJson() as Map<String, dynamic>);
          new ReadyEvent._new(client);
        });
      } else {
        new ReadyEvent._new(client);
      }
    }
  }
}
