part of dartcord;

/// A level of logging.
enum LoggingLevel {
  /// No significance.
  none,

  /// A debug message.
  debug,

  /// Information.
  info,

  /// A stack trace.
  trace,

  /// A general warning.
  warn,

  /// An error.
  err,

  /// A critical error.
  critical,
}

/// Represents a function that logs a value.
typedef void LogHandler(LoggingLevel level, String message);

/// dartcord.dart's logger.
class DiscordLogging {

  /// A map that converts a logging level to a colour.
  static Map<LoggingLevel, String> _colourMap = new Map<LoggingLevel, String>();

  /// A list of functions that logs a value.
  static List<LogHandler> _logHandlers = new List<LogHandler>();

  /// The default logging handler.
  static LogHandler _defaultHandler = (LoggingLevel level, String message) {
    var format = "${(new DateTime.now()).toString()} - ${level.toString()
        .substring(13)
        .toUpperCase()} | ${message}";
    var color = new colorize.Colorize(format);
    color.apply(_colourMap[level]);
    print(color);
  };

  /// Populates the logging level to colour map && populates the log handlers list.
  static void setup() {
    _colourMap[LoggingLevel.none] = colorize.Styles.WHITE;
    _colourMap[LoggingLevel.debug] = colorize.Styles.GREEN;
    _colourMap[LoggingLevel.info] = colorize.Styles.BLUE;
    _colourMap[LoggingLevel.trace] = colorize.Styles.RED;
    _colourMap[LoggingLevel.warn] = _colourMap[LoggingLevel.err] =
    _colourMap[LoggingLevel.critical] = colorize.Styles.RED;
    _logHandlers.add(_defaultHandler);
  }

  /// Logs [message] to console with [level] as the level.
  static void log(LoggingLevel level, String message) {
    for (LogHandler logger in _logHandlers) {
      logger(level, message);
    }
  }

  /// Logs a debug message.
  static void debug(String message) => log(LoggingLevel.debug, message);

  /// Logs an information message.
  static void info(String message) => log(LoggingLevel.info, message);

  /// Logs a basic message.
  static void none(String message) => log(LoggingLevel.none, message);

  /// Logs a stack trace.
  static void trace(String message) => log(LoggingLevel.trace, message);

  /// Logs a warning message.
  static void warn(String message) => log(LoggingLevel.warn, message);

  /// Logs a critical message.
  static void critical(String message) => log(LoggingLevel.critical, message);

  /// Removes the default logger.
  static void removeDefault() {
    _logHandlers.remove(_defaultHandler);
  }

  /// Adds a handler to the list of handlers.
  static void addHandler(LogHandler handler) {
    _logHandlers.add(handler);
  }
}