part of dartcord;

class _GuildAuditLogTargets {
  String guild = 'GUILD';
  String channel = 'CHANNEL';
  String user = 'USER';
  String role = "ROLE";
  String invite = "INVITE";
  String webhook = "WEBHOOK";
  String emoji = "EMOJI";
  String message = "MESSAGE";
  String unknown = "UNKNOWN";
}

class _GuildAuditLogActions {
  Map<String, int> dataMap = <String, int> {
    'GUILD_UPDATE': 1,
    'CHANNEL_CREATE': 10,
    'CHANNEL_UPDATE': 11,
    'CHANNEL_DELETE': 12,
    'CHANNEL_OVERWRITE_CREATE': 13,
    'CHANNEL_OVERWRITE_UPDATE': 14,
    'CHANNEL_OVERWRITE_DELETE': 15,
    'MEMBER_KICK': 20,
    'MEMBER_PRUNE': 21,
    'MEMBER_BAN_ADD': 22,
    'MEMBER_BAN_REMOVE': 23,
    'MEMBER_UPDATE': 24,
    'MEMBER_ROLE_UPDATE': 25,
    'ROLE_CREATE': 30,
    'ROLE_UPDATE': 31,
    'ROLE_DELETE': 32,
    'INVITE_CREATE': 40,
    'INVITE_UPDATE': 41,
    'INVITE_DELETE': 42,
    'WEBHOOK_CREATE': 50,
    'WEBHOOK_UPDATE': 51,
    'WEBHOOK_DELETE': 52,
    'EMOJI_CREATE': 60,
    'EMOJI_UPDATE': 61,
    'EMOJI_DELETE': 62,
    'MESSAGE_DELETE': 72
  };
  int guildUpdate = 1;
  int channelCreate = 10;
  int channelUpdate = 11;
  int channelDelete = 12;
  int channelOverwriteCreate = 13;
  int channelOverwriteUpdate = 14;
  int channelOverwriteDelete = 15;
  int memberKick = 20;
  int memberPrune = 21;
  int memberBanAdd = 22;
  int memberBanRemove = 23;
  int memberUpdate = 24;
  int memberRoleUpdate = 25;
  int roleCreate = 30;
  int roleUpdate = 31;
  int roleDelete = 32;
  int inviteCreate = 40;
  int inviteUpdate = 41;
  int inviteDelete = 42;
  int webhookCreate = 50;
  int webhookUpdate = 51;
  int webhookDelete = 52;
  int emojiCreate = 60;
  int emojiUpdate = 61;
  int emojiDelete = 62;
  int messageDelete = 72;
}

/// Audit logs for a guild, based off of the discord.js implementation.
class GuildAuditLogsContainer extends _Entity {
  /// A list of all entries in this container, mapped by their ID.
  Map<String, GuildAuditLogEntry> auditLogs = new Map<String, GuildAuditLogEntry>();

  /// All audit log targets for this container.
  _GuildAuditLogTargets targets = new _GuildAuditLogTargets();
  /// All audit log actions for this container.
  _GuildAuditLogActions actions = new _GuildAuditLogActions();

  /// Creates a container.
  GuildAuditLogsContainer._new(Client client, Map<String, dynamic> sourceData, Guild guild) : super(client, sourceData) {
    for (var item in sourceData['audit_log_entries']) {
      var entry = new GuildAuditLogEntry._new(this, client, item, guild);
      auditLogs[entry.id] = entry;
    }
  }

  /// Parses an integer to a Target.
  String parseTargetType(int target) {
    if (target < 10) return targets.guild;
    if (target < 20) return targets.channel;
    if (target < 30) return targets.user;
    if (target < 40) return targets.role;
    if (target < 50) return targets.invite;
    if (target < 60) return targets.webhook;
    if (target < 70) return targets.emoji;
    if (target < 80) return targets.message;
    return targets.unknown;
  }

  /// Parses an entry action to a type
  String parseActionType(int input) {
    if ([
      actions.channelCreate,
      actions.channelOverwriteCreate,
      actions.memberBanRemove,
      actions.roleCreate,
      actions.inviteCreate,
      actions.webhookCreate,
      actions.emojiCreate
    ].contains(input)) return 'CREATE';

    if ([
      actions.channelDelete,
      actions.channelOverwriteDelete,
      actions.memberKick,
      actions.memberPrune,
      actions.memberBanAdd,
      actions.roleDelete,
      actions.inviteDelete,
      actions.webhookDelete,
      actions.emojiDelete,
      actions.messageDelete
    ].contains(input)) return 'DELETE';

    if ([
      actions.guildUpdate,
      actions.channelUpdate,
      actions.channelOverwriteUpdate,
      actions.memberUpdate,
      actions.memberRoleUpdate,
      actions.roleUpdate,
      actions.inviteUpdate,
      actions.webhookUpdate,
      actions.emojiUpdate
    ].contains(input)) return 'UPDATE';

    return 'ALL';
  }
}

/// Represents a single guild audit log.
class GuildAuditLogEntry extends _Entity {
  /// The ID of this entry.
  String id;
  /// The target type of this entry.
  String targetType;
  /// The owner of this entry.
  GuildAuditLogsContainer owner;
  /// The action type of this entry.
  String actionType;
  /// The specific action of this entry.
  int action;
  /// The reason of this entry.
  String reason;
  /// The author of this entry.
  Member author;
  /// The specific property changes of this entry.
  List<AuditLogChange> changes;
  /// Creates a guild audit log.
  GuildAuditLogEntry._new(this.owner, Client client, Map<String, dynamic> sourceData, Guild guild) : super(client, sourceData) {
    this.targetType = owner.parseTargetType(sourceData['action_type']);
    this.actionType = owner.parseActionType(sourceData['action_type']);
    this.action = owner.actions.dataMap[sourceData['action_type']];
    this.reason = (sourceData['reason'] == null ? null : sourceData['reason']);
    guild.fetchMember(sourceData['user_id']).then((member) {
      this.author = member;
    });
    this.id = sourceData['id'];
    this.changes = sourceData['changes'] != null ? new List.from(sourceData['changes'].map((c) {
      return new AuditLogChange._new(this, client, c);
    })) : null;
  }
}

/// A specific property change
class AuditLogChange extends _Entity{
  /// The key of this change.
  String key;
  /// The old value of this change.
  String oldValue;
  /// The new value of this change.
  String newValue;
  /// Creates a new audit log change.
  AuditLogChange._new(GuildAuditLogEntry entry, Client client, Map<String, dynamic> sourceData) : super(client, sourceData) {
    key = sourceData['key'];
    oldValue = sourceData['old_value'].toString();
    newValue = sourceData['new_value'].toString();
  }
}